@@ -67,7 +67,7 @@ namespace acommon {
   // Assumes that the maxium number of items in the table is 256
   // Also assumes (unsigned char)i == i % 256
 
-  // Based on the iso8859-* character sets it is very fast, almsot all
+  // Based on the iso-8859-* character sets it is very fast, almsot all
   // lookups involving no more than 2 comparisons.
   // NO looks ups involded more than 3 comparssions.
   // Also, no division (or modules) is done whatsoever.
@@ -175,7 +175,7 @@ namespace acommon {
   // read in char data
   //
 
-  PosibErr<void> read_in_char_data (Config & config,
+  PosibErr<void> read_in_char_data (const Config & config,
 				    ParmString encoding,
 				    ToUniLookup & to,
 				    FromUniLookup & from)
@@ -189,10 +189,9 @@ namespace acommon {
     FStream data;
     PosibErrBase err = data.open(file_name, "r");
     if (err.get_err()) { 
-      String mesg;
-      mesg  = " This could also mean that the file \"";
-      mesg += file_name;
-      mesg += "\" could not be opened for reading or does not exist.";
+      char mesg[128];
+      snprintf(mesg, 128, _("This could also mean that the file \"%s\" could not be opened for reading or does not exist."),
+               file_name.c_str());
       return make_err(unknown_encoding, encoding, mesg);
     }
     unsigned int chr;
@@ -267,7 +266,7 @@ namespace acommon {
   };
 
   template <typename Chr>
-  struct ConvDirect : public Conv
+  struct ConvDirect : public DirectConv
   {
     void convert(const char * in0, int size, CharVector & out) const {
       if (size == -1) {
@@ -288,7 +287,7 @@ namespace acommon {
   struct DecodeLookup : public Decode 
   {
     ToUniLookup lookup;
-    PosibErr<void> init(ParmString code, Config & c) 
+    PosibErr<void> init(ParmString code, const Config & c) 
       {FromUniLookup unused;
       return read_in_char_data(c, code, lookup, unused);}
     void decode(const char * in, int size, FilterCharVector & out) const {
@@ -306,7 +305,7 @@ namespace acommon {
   struct EncodeLookup : public Encode 
   {
     FromUniLookup lookup;
-    PosibErr<void> init(ParmString code, Config & c) 
+    PosibErr<void> init(ParmString code, const Config & c) 
       {ToUniLookup unused;
       return read_in_char_data(c, code, unused, lookup);}
     void encode(const FilterChar * in, const FilterChar * stop, 
@@ -328,11 +327,12 @@ namespace acommon {
   //
   
 #define get_check_next \
-  c = *in;                                                       \
-  if ((c & 0xC0) != 0x80 || in == stop) return FilterChar('?',u);\
-  ++in;                                                          \
-  u <<= 6;                                                       \
-  u |= c & 0x3F;                                                 \
+  if (in == stop) goto error;          \
+  c = *in;                             \
+  if ((c & 0xC0) != 0x80) goto error;  \
+  ++in;                                \
+  u <<= 6;                             \
+  u |= c & 0x3F;                       \
   ++w;
 
   static inline FilterChar from_utf8 (const char * & in, const char * stop)
@@ -343,7 +343,8 @@ namespace acommon {
     // the first char is guaranteed not to be off the end
     char c = *in;
     ++in;
-    while ((c & 0xC0) == 0x80) {c = *in; ++in; ++w;}
+
+    while (in != stop && (c & 0xC0) == 0x80) {c = *in; ++in; ++w;}
     if ((c & 0x80) == 0x00) { // 1-byte wide
       u = c;
     } else if ((c & 0xE0) == 0xC0) { // 2-byte wide
@@ -354,13 +355,30 @@ namespace acommon {
       get_check_next;
       get_check_next;
     } else if ((c & 0xF8) == 0xF0) { // 4-byte wide
-      u  = c & 0x0E;
+      u  = c & 0x07;
       get_check_next;
       get_check_next;
       get_check_next;
+    } else if ((c & 0xFC) == 0xF8) { // 5-byte wide
+      u  = c & 0x03;
+      get_check_next;
+      get_check_next;
+      get_check_next;
+      get_check_next;
+    } else if ((c & 0xFE) == 0xFC) { // 6-byte wide
+      u  = c & 0x03;
+      get_check_next;
+      get_check_next;
+      get_check_next;
+      get_check_next;
+      get_check_next;
+    } else {
+      goto error;
     }
 
     return FilterChar(u, w);
+  error:
+    return FilterChar('?', w);
   }
   
   static inline void to_utf8 (FilterChar in, CharVector & out)
@@ -385,8 +403,23 @@ namespace acommon {
       out.append(0x80 | c>>6 & 0x3F);
       out.append(0x80 | c & 0x3F);
     }
+    else if (c < 0x4000000) {
+      out.append(0xF8 | c>>24);
+      out.append(0x80 | c>>18 & 0x3F);
+      out.append(0x80 | c>>12 & 0x3F);
+      out.append(0x80 | c>>6 & 0x3F);
+      out.append(0x80 | c & 0x3F);
+    }
+    else if (c < 0x80000000) {
+      out.append(0xF8 | c>>30);
+      out.append(0x80 | c>>24 & 0x3F);
+      out.append(0x80 | c>>18 & 0x3F);
+      out.append(0x80 | c>>12 & 0x3F);
+      out.append(0x80 | c>>6 & 0x3F);
+      out.append(0x80 | c & 0x3F);
+    }
   }
-
+  
   struct DecodeUtf8 : public Decode 
   {
     ToUniLookup lookup;
@@ -415,14 +448,13 @@ namespace acommon {
   // new_aspell_convert
   //
 
-  void Convert::generic_convert(const char * in, int size, 
-				CharVector & out)
+  void Convert::generic_convert(const char * in, int size, CharVector & out)
   {
-    buf.clear();
-    decode_->decode(in, size, buf);
-    buf.append(0);
-    FilterChar * start = buf.pbegin();
-    FilterChar * stop = buf.pend();
+    buf_.clear();
+    decode_->decode(in, size, buf_);
+    buf_.append(0);
+    FilterChar * start = buf_.pbegin();
+    FilterChar * stop = buf_.pend();
     if (!filter.empty()) {
       filter.decode(start, stop);
       filter.process(start, stop);
@@ -431,42 +463,61 @@ namespace acommon {
     encode_->encode(start, stop, out);
   }
 
-  PosibErr<Convert *> new_convert(Config & c,
-				  ParmString in, 
-				  ParmString out) 
+  const char * fix_encoding_str(ParmString enc, String & buf)
   {
-    String in_s  = in;
-    String out_s = out;
+    buf.reserve(enc.size() + 1);
+    for (size_t i = 0; i != enc.size(); ++i)
+      buf.push_back(asc_tolower(enc[i]));
 
-    unsigned int i;
-    for (i = 0; i != in_s.size(); ++i)
-      in_s[i] = asc_tolower(in_s[i]);
-    for (i = 0; i != out_s.size(); ++i)
-      out_s[i] = asc_tolower(out_s[i]);
-    in  = in_s .c_str();
-    out = out_s.c_str();
+    if (strncmp(buf.c_str(), "iso8859", 7) == 0)
+      buf.insert(3, 1, '-'); // For backwards compatibility
+    
+    if (buf == "ascii")
+      return "iso-8859-1";
+    else if (buf == "machine unsigned 16")
+      return "utf-16";
+    else if (buf == "machine unsigned 32")
+      return "utf-32";
+    else
+      return buf.c_str();
+  }
 
-    if (in == "ascii") 
-      in = "iso8859-1";
-    if (out == "ascii") 
-      out = "iso8859-1";
+  bool is_ascii_enc(ParmString enc)
+  {
+    return (enc == "ASCII" || enc == "ascii" 
+            || enc == "ANSI_X3.4-1968");
+  }
+
+
+  PosibErr<Convert *> internal_new_convert(const Config & c,
+                                           ParmString in, 
+                                           ParmString out,
+                                           bool if_needed)
+  {
+    String in_s;
+    in = fix_encoding_str(in, in_s);
+
+    String out_s;
+    out = fix_encoding_str(out, out_s); 
+
+    if (if_needed && in == out) return 0;
 
     StackPtr<Convert> conv(new Convert);
     RET_ON_ERR(conv->init(c, in, out));
     return conv.release();
     
   }
-  
-  PosibErr<void> Convert::init(Config & c, ParmString in, ParmString out)
+
+  PosibErr<void> Convert::init(const Config & c, ParmString in, ParmString out)
   {
     in_code_ = in;
     out_code_ = out;
     
-    if (in_code_ == "iso8859-1")
+    if (in_code_ == "iso-8859-1")
       decode_ = new DecodeDirect<Uni8>;
-    else if (in_code_ == "machine unsigned 16")
+    else if (in_code_ == "utf-16")
       decode_ = new DecodeDirect<Uni16>;
-    else if (in_code_ == "machine unsigned 32")
+    else if (in_code_ == "utf-32")
       decode_ = new DecodeDirect<Uni32>;
     else if (in_code_ == "utf-8")
       decode_ = new DecodeUtf8;
@@ -474,11 +525,11 @@ namespace acommon {
       decode_ = new DecodeLookup;
     RET_ON_ERR(decode_->init(in_code_, c));
     
-    if (out_code_ == "iso8859-1")
+    if (out_code_ == "iso-8859-1")
       encode_ = new EncodeDirect<Uni8>;
-    else if (out_code_ == "machine unsigned 16")
+    else if (out_code_ == "utf-16")
       encode_ = new EncodeDirect<Uni16>;
-    else if (out_code_ == "machine unsigned 32")
+    else if (out_code_ == "utf-32")
       encode_ = new EncodeDirect<Uni32>;
     else if (out_code_ == "utf-8")
       encode_ = new EncodeUtf8;
@@ -487,9 +538,9 @@ namespace acommon {
     RET_ON_ERR(encode_->init(out_code_, c));
 
     if (in_code_ == out_code_) {
-      if (in_code_ == "machine unsigned 16")
+      if (in_code_ == "utf-16")
 	conv_ = new ConvDirect<Uni16>;
-      else if (in_code_ == "machine unsigned 32")
+      else if (in_code_ == "utf-32")
 	conv_ = new ConvDirect<Uni32>;
       else
 	conv_ = new ConvDirect<char>;
