@@ -219,7 +219,7 @@ namespace acommon {
     FixedBuffer<> buf; DataPair d;
     while (getdata_pair(in, d, buf)) {
       if (d.key == "order-num") {
-	to_add->c_struct.order_num = strtod_c(d.value.str(), NULL);
+	to_add->c_struct.order_num = strtod_c(d.value.str, NULL);
 	if (!(0 < to_add->c_struct.order_num && 
 	      to_add->c_struct.order_num < 1)) 
 	  {
@@ -228,7 +228,7 @@ namespace acommon {
 	    goto RETURN_ERROR;
 	  }
       } else if (d.key == "lib-dir") {
-	to_add->lib_dir = d.value.str();
+	to_add->lib_dir = d.value.str;
 	to_add->c_struct.lib_dir = to_add->lib_dir.c_str();
       } else if (d.key == "dict-dir" || d.key == "dict-dirs") {
 	to_add->c_struct.dict_dirs = &(to_add->dict_dirs);
