@@ -15,6 +15,7 @@
 #include "wordinfo.hpp"
 #include "fstream.hpp"
 #include "parm_string.hpp"
+#include "simple_string.hpp"
 #include "char_vector.hpp"
 #include "objstack.hpp"
 
@@ -37,6 +38,7 @@ namespace aspeller {
   using namespace acommon;
 
   class Language;
+  class Conv;
 
   class SpellerImpl;
   using acommon::CheckInfo;
@@ -54,8 +56,9 @@ namespace aspeller {
 
   struct WordAff
   {
-    String word;
-    String af;
+    SimpleString word;
+    const unsigned char * aff;
+    WordAff * next;
   };
 
   class AffixMgr
@@ -67,6 +70,9 @@ namespace aspeller {
     PfxEntry *          pFlag[SETSIZE];
     SfxEntry *          sFlag[SETSIZE];
 
+    int max_strip_f[SETSIZE];
+    int max_strip_;
+
     const char *        encoding;
     //const char *        compound;
     //int                 cpdmin;
@@ -81,7 +87,9 @@ namespace aspeller {
     AffixMgr(const Language * l) : lang(l), data_(0) {}
     ~AffixMgr();
 
-    PosibErr<void> setup(ParmString affpath);
+    unsigned int max_strip() const {return max_strip_;}
+
+    PosibErr<void> setup(ParmString affpath, Conv &);
 
     bool affix_check(const LookupInfo &, ParmString, CheckInfo &, GuessInfo *) const;
     bool prefix_check(const LookupInfo &, ParmString, CheckInfo &, GuessInfo *) const;
@@ -90,16 +98,24 @@ namespace aspeller {
 
     void get_word(String & word, const CheckInfo &) const;
 
-    void  munch(ParmString word, CheckList *) const;
-    void  expand(ParmString word, ParmString affixes, CheckList *) const;
-    // expand enough so the affixes does not effect the first limit
-    // characters
-    int  expand(ParmString word, ParmString af, int limit, WordAff * l) const;
+    void munch(ParmString word, CheckList *) const;
 
+    WordAff * expand(ParmString word, ParmString aff, 
+                     ObjStack &, int limit = INT_MAX) const;
+
+    WordAff * expand_prefix(ParmString word, ParmString aff, 
+                            ObjStack & buf) const 
+    {
+      return expand(word,aff,buf,0);
+    }
+    WordAff * expand_suffix(ParmString word, const unsigned char * new_aff,
+                            ObjStack &, int limit = INT_MAX,
+                            unsigned char * new_aff = 0, WordAff * * * l = 0) const;
+    
   private:
-    PosibErr<void> parse_file(const char * affpath);
+    PosibErr<void> parse_file(const char * affpath, Conv &);
 
-    void encodeit(AffEntry * ptr, char * cs);
+    void encodeit(AffEntry * ptr, const char * cs);
     PosibErr<void> build_pfxlist(PfxEntry* pfxptr);
     PosibErr<void> build_sfxlist(SfxEntry* sfxptr);
     PosibErr<void> process_pfx_order();
@@ -107,8 +123,8 @@ namespace aspeller {
   };
 
   PosibErr<AffixMgr *> new_affix_mgr(ParmString name, 
+                                     Conv &,
                                      const Language * lang);
-
 }
 
 #endif
