@@ -36,10 +36,12 @@
 // FIXME: OPTIMIZATIONS
 //   store the number of letters that are the same as the previous 
 //     soundslike so that it can possible be skipped
-//   don't expand every affix immendently as the whole entry may be
-//     able to be skipped based on the scan of the root word.
-//     this will require storing the maxim number of words stripped
-//     with each entry
+// FIXME: Improve Suggest
+//   after scoring and ranking the suggestions there are not enough
+//     words than resort to MySpell ngram
+//   also considering first aiming for one edit distance then if that
+//     is not enough aim for two.  This strategy should only be used
+//     when true soundslike are not used.
 
 #include "getdata.hpp"
 
@@ -443,8 +445,10 @@ namespace {
     String sl_buf;
     EditDist score;
     unsigned int stopped_at = LARGE_NUM;
-    CheckList cl;
-    const CheckInfo * ci_cur;
+    ObjStack exp_buf;
+    WordAff * exp_list;
+    WordAff single;
+    single.next = 0;
     
     for (SpellerImpl::WS::const_iterator i = speller->suggest_ws.begin();
          i != speller->suggest_ws.end();
@@ -488,19 +492,63 @@ namespace {
 
       affix_case:
 
-        lang->affix()->expand(sw->word, sw->aff, &cl);
-        ci_cur = cl.data + 1;
-        for (;ci_cur; ci_cur = ci_cur->next) {
+        exp_buf.reset();
+
+        // FIXME: enhance expand to only used stripped words and
+        //        prefixes so that only the root word needs to be
+        //        stripped.  BUT this could cause a problem if there
+        //        is a different expansion depending on the accent
+
+        // first expand any prefixes
+        if (fast_scan) { // if fast_scan than no prefixes
+          single.word.str = sw->word;
+          single.word.size = strlen(sw->word);
+          single.aff = (const unsigned char *)sw->aff;
+          exp_list = &single;
+        } else {
+          exp_list = lang->affix()->expand_prefix(sw->word, sw->aff, exp_buf);
+        }
+
+        // iterate through each semi-expanded word, any affix flags
+        // are now guaranteed to be suffixes
+        for (WordAff * p = exp_list; p; p = p->next)
+        {
+          // try the root word
           sl_buf.clear();
-          to_stripped(*lang, ci_cur->word, sl_buf);
+          to_stripped(*lang, p->word, sl_buf);
           score = edit_dist_fun(sl_buf.c_str(), original_soundslike, parms.edit_distance_weights);
-          stopped_at = score.stopped_at - sl;
-          if (score >= LARGE_NUM) continue;
-          stopped_at = LARGE_NUM;
-          ParmString w_word(ci_cur->word);
-          char * wf = (char *)buffer.alloc(w_word.size() + 1);
-          i->convert.convert(w_word, wf);
-          add_nearmiss(ParmString(wf, w_word.size()), score, do_count);
+          stopped_at = score.stopped_at - sl_buf.c_str();
+
+          if (score < LARGE_NUM) {
+            char * wf = (char *)buffer.alloc(p->word.size + 1);
+            i->convert.convert(p->word, wf);
+            add_nearmiss(ParmString(wf, p->word.size), score, do_count);
+          }
+
+          // expand any suffixes, using stopped_at as a hint to avoid
+          // unneeded expansions.  Note stopped_at is the last character
+          // looked at by limit_edit_dist.  Thus if the character
+          // at stopped_at is changed it might effect the result
+          // hence the "limit" is stopped_at + 1
+          if (p->word.size - lang->affix()->max_strip() > stopped_at)
+            exp_list = 0;
+          else
+            exp_list = lang->affix()->expand_suffix(p->word, p->aff, 
+                                                    exp_buf, stopped_at + 1);
+
+          // reset stopped_at if necessary
+          if (score < LARGE_NUM) stopped_at = LARGE_NUM;
+
+          // iterate through fully expanded words, if any
+          for (WordAff * q = exp_list; q; q = q->next) {
+            sl_buf.clear();
+            to_stripped(*lang, q->word, sl_buf);
+            score = edit_dist_fun(sl_buf.c_str(), original_soundslike, parms.edit_distance_weights);
+            if (score >= LARGE_NUM) continue;
+            char * wf = (char *)buffer.alloc(q->word.size + 1);
+            i->convert.convert(q->word, wf);
+            add_nearmiss(ParmString(wf, q->word.size), score, do_count);
+          }
         }
       }
     }
@@ -915,7 +963,7 @@ namespace aspeller {
       FixedBuffer<64> buf;
       DataPair d;
       while (getdata_pair(in, d, buf)) {
-	if (d.key.size() != 2)
+	if (d.key.size != 2)
 	  return make_err(bad_file_format, file);
 	w.repl (l.to_normalized(d.key[0]),
 		l.to_normalized(d.key[1])) = w.repl_dis1;
